﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace IGStockAnnotator.ViewModel
{
    public enum EventSource
    {
        Economy,
        Company
    }

    /// <summary>
    /// Represents an annotation of an event
    /// </summary>
    public class EventAnnotation : INotifyPropertyChanged
    {

        private DateTime _timestamp;
        private string _description, _userDescription, _details;

        /// <summary>
        /// Detailed information about the event
        /// </summary>
        public string Details
        {
            get
            {
                return "Details for timestamp:" +
                  this.Timestamp +
                  "\nDescription:" + this.Description +
                  "\nUser description:" + this.UserDescription +
                  "\n\nThis is where detailed information about the event can be provided";
            }
        }
        private EventSource _eventSource;

        /// <summary>
        /// Type of event
        /// </summary>
        public EventSource EventSource
        {
            get { return _eventSource; }
            set 
            { 
                _eventSource = value;
                RaisePropertyChanged("EventSource");
            }
        }
        
        /// <summary>
        /// Indicates when the event took place
        /// </summary>
        public DateTime Timestamp
        {
            get { return _timestamp; }
            set 
            { 
                _timestamp = value;
                RaisePropertyChanged("Timestamp");
            }
        }
        
        /// <summary>
        /// Description of the event
        /// </summary>
        public string Description
        {
            get
            {
                return _description;
            }
            set
            {
                _description = value;
                RaisePropertyChanged("Description");
            }
        }
        
        /// <summary>
        /// User-provided additional description of the event
        /// </summary>
        public string UserDescription
        {
            get
            {
                return _userDescription;
            }
            set
            {
                _userDescription = value;
                RaisePropertyChanged("UserDescription");
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
        public void RaisePropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        /// <summary>
        /// Generates a set of sample event annotations
        /// </summary>
        /// <param name="initialTime">Starting time for the generated events</param>
        /// <returns></returns>
        public static ObservableBindingList<EventAnnotation> Generate(DateTime initialTime)
        {
            ObservableBindingList<EventAnnotation> list = new ObservableBindingList<EventAnnotation>();
            Random rand = new Random(DateTime.Now.Millisecond);
            DateTime timestamp = initialTime;
            bool areInterestRatesGoingUp = false;
            bool isECB = false;

            while (timestamp < DateTime.Now)
            {
                EventAnnotation ea = new EventAnnotation() { Timestamp = timestamp };

                if (rand.Next() % 2 == 1)
                {
                    ea.EventSource = ViewModel.EventSource.Economy;
                        ea.Description = areInterestRatesGoingUp ?
                        isECB ? "The ECB has increased interest rates" : "The Federal Reserve has increased interest rates" :
                        isECB ? "The ECB has decreased interest rates" : "The Federal Reserve has decreased interest rates";
                    
                    areInterestRatesGoingUp = rand.Next() % 2 == 1;
                    isECB = rand.Next() % 2 == 1;
                }
                else
                {
                    ea.Timestamp = timestamp;
                    ea.EventSource = ViewModel.EventSource.Company;
                    ea.Description = "Announced projected earnings for Q" + Convert.ToString(timestamp.Month % 4 + 1);
                }

                list.Add(ea);
                timestamp = timestamp.AddDays(rand.Next(50));
                
                if (timestamp > DateTime.Now)
                    timestamp = DateTime.Now;
            }
            return list;
        }
    }
}
